@@ -10,6 +10,8 @@ class TestFixCliOption(BaseTestCase):
             "ignore_names_by_pattern.py": """
                 class UnusedClass:
                     pass
+
+                print("Keep the file")
                 """
         }
 
@@ -17,31 +19,42 @@ class TestFixCliOption(BaseTestCase):
         self.assertEqual(
             unused_names,
             (
-                "ignore_names_by_pattern.py:1:0: DC100 Global UnusedClass is never used\n\n"
-                "Removed \x1b[1m1\x1b[0m unused code item!"
+                "ignore_names_by_pattern.py:1:0: DC03 Class `UnusedClass` is never used\n\n"
+                "Removed 1 unused code item!"
             ),
         )
 
-        self.assertFiles({"ignore_names_by_pattern.py": """"""})
+        self.assertFiles(
+            {
+                "ignore_names_by_pattern.py": """
+            print("Keep the file")
+        """
+            }
+        )
 
     def test_function_removal(self):
         self.files = {
             "ignore_names_by_pattern.py": """
                 def foo(bar: str = "Bar") -> str:
                     return 1 ** 2
+
+                print("Keep the file")
                 """
         }
 
         unused_names = main(["ignore_names_by_pattern.py", "--no-color", "--fix"])
         self.assertEqual(
             unused_names,
-            (
-                "ignore_names_by_pattern.py:1:0: DC100 Global foo is never used\n\n"
-                "Removed \x1b[1m1\x1b[0m unused code item!"
-            ),
+            ("ignore_names_by_pattern.py:1:0: DC02 Function `foo` is never used\n\n" "Removed 1 unused code item!"),
         )
 
-        self.assertFiles({"ignore_names_by_pattern.py": """"""})
+        self.assertFiles(
+            {
+                "ignore_names_by_pattern.py": """
+                print("Keep the file")
+        """
+            }
+        )
 
     # TODO: treat variables unused if they are referenced only by other unused code.
 
@@ -61,10 +74,7 @@ class TestFixCliOption(BaseTestCase):
         unused_names = main(["ignore_names_by_pattern.py", "--no-color", "--fix"])
         self.assertEqual(
             unused_names,
-            (
-                "ignore_names_by_pattern.py:3:0: DC100 Global foo is never used\n\n"
-                "Removed \x1b[1m1\x1b[0m unused code item!"
-            ),
+            ("ignore_names_by_pattern.py:3:0: DC02 Function `foo` is never used\n\n" "Removed 1 unused code item!"),
         )
 
         self.assertFiles(
@@ -97,9 +107,9 @@ class TestFixCliOption(BaseTestCase):
         self.assertEqual(
             unused_names,
             (
-                "ignore_names_by_pattern.py:3:0: DC100 Global unused_variable is never used\n"
-                "ignore_names_by_pattern.py:5:0: DC100 Global unused_function is never used\n\n"
-                "Removed \x1b[1m2\x1b[0m unused code items!"
+                "ignore_names_by_pattern.py:3:0: DC01 Variable `unused_variable` is never used\n"
+                "ignore_names_by_pattern.py:5:0: DC02 Function `unused_function` is never used\n\n"
+                "Removed 2 unused code items!"
             ),
         )
 
@@ -503,9 +513,16 @@ class TestKeepCorrectNumberOfEmptyLinesAfterRemovalOfCodeBlock(BaseTestCase):
         self.files = {
             "foo.py": """
             bar = 1
+            print("Keep the file")
             """
         }
 
         main([".", "--fix"])
 
-        self.assertFiles({"foo.py": """"""})
+        self.assertFiles(
+            {
+                "foo.py": """
+            print("Keep the file")
+        """
+            }
+        )
