@@ -4,6 +4,7 @@ import os
 import re
 import string
 import sys
+from logging import getLogger
 
 from pathlib import Path
 
@@ -14,6 +15,7 @@ from deadcode.visitor.code_item import CodeItem
 from deadcode.visitor.utils import LoggingList, LoggingSet
 from deadcode.visitor import utils
 from deadcode.actions.parse_abstract_syntax_tree import parse_abstract_syntax_tree
+from deadcode.utils.nested_scopes import NestedScope
 
 from deadcode.visitor import noqa
 from deadcode.utils.print_ast import print_ast as show  # noqa: F401
@@ -32,6 +34,8 @@ from deadcode.visitor.ignore import (
     _ignore_variable,
 )
 
+logger = getLogger()
+
 
 class DeadCodeVisitor(ast.NodeVisitor):
     """Finds dead code."""
@@ -70,89 +74,23 @@ class DeadCodeVisitor(ast.NodeVisitor):
         self.should_ignore_new_definitions = False
 
         self.noqa_lines: Dict[str, Set[int]] = {}
+        self.scopes = NestedScope()
 
     @property
     def scope(self) -> str:
         return ".".join(self.scope_parts)
 
     def add_used_name(self, name: str, scope: Optional[str] = None) -> None:
+        # TODO: Usage should be tracked on CodeItem.
+
+        # TODO: Lets first resolve, how to correctly set the type of a function argument.
+        # We should have CodeItem as a type, not only name as string.
+        # TODO: investigate: am I always able to retrieve the type of a variable?
+
         self.used_names.add(name)
-
-    # def scan(self, code, filename=""):
-    #     filename = Path(filename)
-    #     self.code = code.splitlines()
-    #     self.noqa_lines = noqa.parse_noqa(self.code)
-    #     self.filename = filename
-
-    #     def handle_syntax_error(e):
-    #         text = f' at "{e.text.strip()}"' if e.text else ""
-    #         self._log(
-    #             f"{utils.format_path(filename)}:{e.lineno}: {e.msg}{text}",
-    #             file=sys.stderr,
-    #             force=True,
-    #         )
-    #         self.found_dead_code_or_error = True
-
-    #     try:
-    #         node = ast.parse(code, filename=str(self.filename), type_comments=True)
-    #     except SyntaxError as err:
-    #         handle_syntax_error(err)
-    #     except ValueError as err:
-    #         # ValueError is raised if source contains null bytes.
-    #         self._log(
-    #             f'{utils.format_path(filename)}: invalid source code "{err}"',
-    #             file=sys.stderr,
-    #             force=True,
-    #         )
-    #         self.found_dead_code_or_error = True
-    #     else:
-    #         # When parsing type comments, visiting can throw SyntaxError.
-    #         try:
-    #             self.visit(node)
-    #         except SyntaxError as err:
-    #             handle_syntax_error(err)
-
-    # TODO: do not load the AST of all the files into memory.
-    # Should instead parse the files one by one and discard already parsed files.
-    # The memory might add up for larger projects, especially that AST is whay larger the the code itself.
+        self.scopes.mark_as_used(name, self.scope)
 
     def visit_abstract_syntax_trees(self) -> None:
-        # NEXTODO:
-        # So whats here? These examples could be moved to exclusion before.
-        # Also AST should not be constructed until the parsing part.
-
-        # def prepare_pattern(pattern):
-        #     if not any(char in pattern for char in "*?["):
-        #         pattern = f"*{pattern}*"
-        #     return pattern
-
-        # exclude = [prepare_pattern(pattern) for pattern in (self.args.exclude or [])]
-
-        # def exclude_path(path):
-        #     return _match(path, exclude, case=False)
-
-        # # paths = [Path(path) for path in paths]
-
-        # for module in utils.get_modules(paths):
-        #     if exclude_path(module):
-        #         self._log("Excluded:", module)
-        #         continue
-
-        # self._log("Scanning:", module)
-        # try:
-        #     module_string = utils.read_file(module)
-        # except utils.VultureInputException as err:  # noqa: F841
-        #     self._log(
-        #         f"Error: Could not read file {module} - {err}\n"
-        #         f"Try to change the encoding to UTF-8.",
-        #         file=sys.stderr,
-        #         force=True,
-        #     )
-        #     self.found_dead_code_or_error = True
-        # else:
-        #     self.scan(module_string, filename=module)
-
-        # TODO: should parse the AST here.
         for file_path in self.filenames:
             with open(file_path) as f:
                 filename = os.path.basename(file_path)
@@ -176,22 +114,7 @@ class DeadCodeVisitor(ast.NodeVisitor):
                         )
                     )
 
-        # unique_imports = {item.name for item in self.defined_imports}
-        # for import_name in unique_imports:
-        #     path = Path("whitelists") / (import_name + "_whitelist.py")
-        #     if exclude_path(path):
-        #         self._log("Excluded whitelist:", path)
-        #     else:
-        #         try:
-        #             module_data = pkgutil.get_data("vulture", str(path))
-        #             self._log("Included whitelist:", path)
-        #         except OSError:
-        #             # Most imported modules don't have a whitelist.
-        #             continue
-        #         module_string = module_data.decode("utf-8")
-        #         self.scan(module_string, filename=path)
-
-    def get_unused_code_items(self, sort_by_size: bool = False) -> Iterable[CodeItem]:
+    def get_unused_code_items(self) -> Iterable[CodeItem]:
         """
         Return ordered list of unused CodeItem objects.
         """
@@ -204,25 +127,12 @@ class DeadCodeVisitor(ast.NodeVisitor):
             self.unused_methods,
             self.unused_props,
             self.unused_vars,
-            self.unreachable_code,
+            # self.unreachable_code,  # TODO: removal of unreachable_code has a lot of edge cases
             self.unused_file,
         )
 
         return sorted(unused_code, key=lambda item: (item.filename, item.name_line or 0))
 
-    # TODO: investigate whitelisting options
-    # def report(self, sort_by_size=False, make_whitelist=False):
-    #     """
-    #     Print ordered list of CodeItem objects to stdout.
-    #     """
-    #     for item in self.get_unused_code_items(sort_by_size=sort_by_size):
-    #         self._log(
-    #             item.get_whitelist_string() if make_whitelist else item.get_report(add_size=sort_by_size),
-    #             force=True,
-    #         )
-    #         self.found_dead_code_or_error = True
-    #     return self.found_dead_code_or_error
-
     @property
     def unused_classes(self) -> Iterable[CodeItem]:
         return _get_unused_items(self.defined_classes, self.used_names)
@@ -266,6 +176,8 @@ class DeadCodeVisitor(ast.NodeVisitor):
         We delegate to this method instead of using visit_alias() to have
         access to line numbers and to filter imports from __future__.
         """
+        # TODO: store full module path. Consider relative and absolute module import collisions.
+
         assert isinstance(node, (ast.Import, ast.ImportFrom))
         for name_and_alias in node.names:
             # Store only top-level module name ("os.path" -> "os").
@@ -345,23 +257,26 @@ class DeadCodeVisitor(ast.NodeVisitor):
         type_: UnusedCodeType = collection.type_  # type: ignore
         first_lineno = lines.get_first_line_number(first_node)
 
+        last_lineno = lines.get_last_line_number(last_node)
+
+        inherits_from = getattr(first_node, "inherits_from", None)
+        code_item = CodeItem(
+            name=name,
+            type_=type_,
+            filename=self.filename,
+            code_parts=[Part(first_lineno, last_lineno, last_node.col_offset, last_node.end_col_offset or 0)],
+            scope=self.scope,
+            name_line=last_node.lineno,  # TODO: Maybe this should be a property?
+            name_column=last_node.col_offset,  # TODO: Maybe this should be a property?
+            message=message,
+            inherits_from=inherits_from,
+        )
+        self.scopes.add(code_item)
+
         if ignored(first_lineno, type_=type_):
             self._log(f'Ignoring {type_} "{name}"')
         else:
-            last_lineno = lines.get_last_line_number(last_node)
-
-            collection.append(
-                CodeItem(
-                    name=name,
-                    type_=type_,
-                    filename=self.filename,
-                    code_parts=[Part(first_lineno, last_lineno, last_node.col_offset, last_node.end_col_offset or 0)],
-                    scope=self.scope,
-                    name_line=last_node.lineno,  # TODO: Maybe this should be a property?
-                    name_column=last_node.col_offset,  # TODO: Maybe this should be a property?
-                    message=message,
-                )
-            )
+            collection.append(code_item)
 
     def _define_variable(self, name: str, node: ast.AST) -> None:
         self._define(
@@ -443,6 +358,22 @@ class DeadCodeVisitor(ast.NodeVisitor):
             and not node.keywords
         )
 
+    def get_inherits_from(self, node: ast.AST) -> Optional[List[str]]:
+        """Returns a set of base classes from any level in inheritance tree."""
+
+        if not (bases_attr := getattr(node, "bases", None)):
+            return None
+
+        bases = [base.id for base in bases_attr if getattr(base, "id", None)]
+        inherits_from = bases[:]
+
+        for base in bases:
+            if base_code_item := self.scopes.get(name=base, scope=self.scope):
+                if base_inherits_from := getattr(base_code_item, "inherits_from", None):
+                    inherits_from.extend(base_inherits_from)
+
+        return inherits_from
+
     def visit_ClassDef(self, node: ast.ClassDef) -> None:
         for decorator in node.decorator_list:
             if _match(utils.get_decorator_name(decorator), self.ignore_decorators):  # type: ignore
@@ -507,6 +438,39 @@ class DeadCodeVisitor(ast.NodeVisitor):
             self.add_used_name(kwd_attr)
 
     def visit(self, node: ast.AST) -> None:
+        node_name = getattr(node, "name", None)
+
+        # > TODO: Expr scope has to be updated after visiting nested nodes.
+        # Do I have to merge custom visits with visit method in order to achieve this?
+        #
+        # I should write a unit test with only an exrepssion and check which methods are being called.
+        # I should only handled that single expression handling properly.
+
+        # breakpoint()
+
+        if inherits_from := self.get_inherits_from(node):
+            node.inherits_from = inherits_from  # type: ignore
+
+        should_turn_off_ignore_new_definitions = False
+        if (
+            # Name is in ignore_definitions
+            node_name
+            and _match(node_name, self.args.ignore_definitions)
+        ) or (
+            # Class inherits from ignore_definitions_if_inherits_from
+            inherits_from
+            and _match_many(inherits_from, self.args.ignore_definitions_if_inherits_from)
+        ):
+            if not self.should_ignore_new_definitions:
+                self.should_ignore_new_definitions = True
+                should_turn_off_ignore_new_definitions = True
+
+        method_name = "visit_" + node.__class__.__name__
+        visitor = getattr(self, method_name, None)
+
+        if visitor:
+            visitor(node)
+
         # TODO: use decorator for this code chunk
         was_scope_increased = True
         if isinstance(node, ast.ClassDef):
@@ -516,39 +480,8 @@ class DeadCodeVisitor(ast.NodeVisitor):
         else:
             was_scope_increased = False
 
-        # TODO: use decorator for this code chunk
-        should_turn_off_ignore_new_definitions = False
-
-        node_name = getattr(node, "name", None)
-        bases_attr = getattr(node, "bases", [])
-        bases = [base.id for base in bases_attr if getattr(base, "id", None)]
-
-        if (
-            # Name is in ignore_definitions
-            node_name
-            and _match(node_name, self.args.ignore_definitions)
-        ) or (
-            # Class inherits from ignore_definitions_if_inherits_from
-            bases_attr
-            and bases
-            and _match_many(bases, self.args.ignore_definitions_if_inherits_from)
-        ):
-            if not self.should_ignore_new_definitions:
-                self.should_ignore_new_definitions = True
-                should_turn_off_ignore_new_definitions = True
-
-        method_name = "visit_" + node.__class__.__name__
-        visitor = getattr(self, method_name, None)
-        if self.verbose:
-            lineno = getattr(node, "lineno", 1)
-            line = self.code[lineno - 1] if self.code else ""
-            self._log(lineno, ast.dump(node), line)
-
-        if visitor:
-            visitor(node)
-
         # Class inherits from ignore_bodies_if_inherits_from
-        if bases and _match_many(bases, self.args.ignore_bodies_if_inherits_from):
+        if inherits_from and _match_many(inherits_from, self.args.ignore_bodies_if_inherits_from):
             if not self.should_ignore_new_definitions:
                 self.should_ignore_new_definitions = True
                 should_turn_off_ignore_new_definitions = True
@@ -560,7 +493,19 @@ class DeadCodeVisitor(ast.NodeVisitor):
             mode = "func_type" if isinstance(node, (ast.FunctionDef, ast.AsyncFunctionDef)) else "eval"
             self.visit(ast.parse(type_comment, filename="<type_comment>", mode=mode))
 
-        self.generic_visit(node)
+        ###########
+        # """Called if no explicit visitor function exists for a node."""
+        # TODO: for assignment statements unused_code unit should be whole assignment statement instead of a name itself
+        # Note: Node is None if file is empty, contains only a docstring, a comment or has a SyntaxError.
+        if node:
+            for _, value in ast.iter_fields(node):
+                if isinstance(value, list):
+                    self._handle_ast_list(value)
+                    for item in value:
+                        if isinstance(item, ast.AST):
+                            self.visit(item)
+                elif isinstance(value, ast.AST):
+                    self.visit(value)
 
         # TODO: use decorator for this code chunk
         if should_turn_off_ignore_new_definitions:
@@ -589,20 +534,3 @@ class DeadCodeVisitor(ast.NodeVisitor):
                     message=f"unreachable code after '{class_name}'",
                 )
                 return None
-
-    def generic_visit(self, node: ast.AST) -> None:
-        """Called if no explicit visitor function exists for a node."""
-        # TODO: for assignment statements unused_code unit should be whole assignment statement instead of a name itself
-
-        # Note: Node is None if file is empty, contains only a docstring, a comment or has a SyntaxError.
-        if not node:
-            return None
-
-        for _, value in ast.iter_fields(node):
-            if isinstance(value, list):
-                self._handle_ast_list(value)
-                for item in value:
-                    if isinstance(item, ast.AST):
-                        self.visit(item)
-            elif isinstance(value, ast.AST):
-                self.visit(value)
