@@ -16,9 +16,34 @@ NOQA_REGEXP = re.compile(
 
 NOQA_CODE_MAP = {
     # flake8 F401: module imported but unused.
-    "F401": "V104",
+    "F401": "DC07",
     # flake8 F841: local variable is assigned to but never used.
-    "F841": "V107",
+    "F841": "DC01",
+    "DC01": "DC01",
+    "DC02": "DC02",
+    "DC03": "DC03",
+    "DC04": "DC04",
+    "DC05": "DC05",
+    "DC06": "DC06",
+    "DC07": "DC07",
+    "DC08": "DC08",
+    "DC09": "DC09",
+    "DC11": "DC11",
+    "DC12": "DC12",
+    "DC13": "DC13",
+    # For backward compatibility
+    "DC001": "DC01",
+    "DC002": "DC02",
+    "DC003": "DC03",
+    "DC004": "DC04",
+    "DC005": "DC05",
+    "DC006": "DC06",
+    "DC007": "DC07",
+    "DC008": "DC08",
+    "DC009": "DC09",
+    "DC011": "DC11",
+    "DC012": "DC12",
+    "DC013": "DC13",
 }
 
 
@@ -29,7 +54,7 @@ def _parse_error_codes(matches_dict: Dict[str, str]) -> List[str]:
 
 def parse_noqa(code: str) -> Dict[str, Set[int]]:
     noqa_lines = defaultdict(set)
-    for lineno, line in enumerate(code, start=1):
+    for lineno, line in enumerate(code.split("\n"), start=1):
         match = NOQA_REGEXP.search(line)
         if match:
             for error_code in _parse_error_codes(match.groupdict()):
